/*
 * Copyright (C) 2020 Jolla Ltd.
 * Copyright (C) 2020 Slava Monich <slava.monich@jolla.com>
 *
 * You may use this file under the terms of BSD license as follows:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *   3. Neither the names of the copyright holders nor the names of its
 *      contributors may be used to endorse or promote products derived
 *      from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef NFC_PEER_SERVICE_PRIVATE_H
#define NFC_PEER_SERVICE_PRIVATE_H

#include "nfc_types_p.h"

#include <nfc_peer_service.h>

NfcPeerConnection*
nfc_peer_service_new_connect(
    NfcPeerService* service,
    guint8 rsap,
    const char* rname)
    NFCD_INTERNAL;

NfcPeerConnection*
nfc_peer_service_new_accept(
    NfcPeerService* service,
    guint8 rsap)
    NFCD_INTERNAL;

void
nfc_peer_service_connection_created(
    NfcPeerService* service,
    NfcPeerConnection* connection)
    NFCD_INTERNAL;

void
nfc_peer_service_connection_dead(
    NfcPeerService* service,
    NfcPeerConnection* connection)
    NFCD_INTERNAL;

void
nfc_peer_service_peer_arrived(
    NfcPeerService* service,
    NfcPeer* peer)
    NFCD_INTERNAL;

void
nfc_peer_service_peer_left(
    NfcPeerService* service,
    NfcPeer* peer)
    NFCD_INTERNAL;

void
nfc_peer_service_datagram_received(
    NfcPeerService* service,
    guint8 ssap,
    const void* data,
    guint len)
    NFCD_INTERNAL;

#endif /* NFC_PEER_SERVICE_PRIVATE_H */

/*
 * Local Variables:
 * mode: C
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 */
